/*

  MICROCONTROL.C - (C) 2004 VLSI SOLUTION OY

  Example program functions which activates VS1002d's ADPCM quality patch, then
  reads ADPCM data from VS1002d and stores it to a file.

  This file should be helpful when creating your own ADPCM data. The
  example file in this directory, adpcm_ex.wav was created with the
  help of these functions.

  Disclaimer: Many necessary error checks have been omitted for clarity.
  No warranty whatsoever.

*/


#define SCI_DATA SCI_HDAT0
#define SCI_BUF_FILL SCI_HDAT1

unsigned char header[] = {
    0x52, 0x49, 0x46, 0x46, 0x1c, 0x10, 0x00, 0x00,
    0x57, 0x41, 0x56, 0x45, 0x66, 0x6d, 0x74, 0x20, /*|RIFF....WAVEfmt |*/
    0x14, 0x00, 0x00, 0x00, 0x11, 0x00, 0x01, 0x00,
    0x40, 0x1f, 0x00, 0x00, 0xd7, 0x0f, 0x00, 0x00, /*|........@......|*/
    0x00, 0x01, 0x04, 0x00, 0x02, 0x00, 0xf9, 0x01,
    0x66, 0x61, 0x63, 0x74, 0x04, 0x00, 0x00, 0x00, /*|.......fact....|*/
    0x5c, 0x1f, 0x00, 0x00, 0x64, 0x61, 0x74, 0x61,
    0xe8, 0x0f, 0x00, 0x00
};


FILE *imaFp = NULL;

#define BLOCK_ALIGN 256
#define SAMPLES_PER_BLOCK (BLOCK_ALIGN*2-7)

void WriteLe(unsigned char *d, unsigned long n) {
  *d++ = n & 0xFF;
  n >>= 8;
  *d++ = n & 0xFF;
  n >>= 8;
  *d++ = n & 0xFF;
  n >>= 8;
  *d = n & 0xFF;
}


/*

  This routine gets called when CTRL-C is pushed

*/
void UpdateHeader(int i) {
  unsigned long len = ftell(imaFp)-sizeof(header);

  WriteLe(header+48, len/BLOCK_ALIGN*SAMPLES_PER_BLOCK);
  WriteLe(header+56, len);
  WriteLe(header+ 4, len+52);

  /* Jump back to beginning of file and update header */
  fseek(imaFp, 0, SEEK_SET);
  fwrite(header, 1, sizeof(header), imaFp);
  fclose(imaFp);

  exit(EXIT_SUCCESS);
}



/*

  This is the main recording function.

*/
void RecordAdpcm(void) {
  unsigned char buf[256];
  int bufPos=0, t, readWords;
  imaFp = fopen("adpcm.wav", "wb");

  /* Write header although some values are still missing */
  fwrite(buf, 1, sizeof(header), imaFp);
  /* Catch CTRL-C so that UpdateHeader is called */
  signal(SIGINT, UpdateHeader);

  SendCmd(SCI_MODE,    0xC84);		/* Normal SW reset + other bits */
  WaitDreq();				/* Wait for DREQ to go up */
  SendCmd(SCI_AICTRL0, 0xC);		/* Sample rate */
  SendCmd(SCI_AICTRL1, 0x0);		/* Auto gain */
  SendCmd(SCI_MODE,    0x1C84);		/* ADPCM SW reset + other bits */
  WaitDreq();				/* Wait for DREQ to go up */
  /*
    Even if your clock rate is not exactly 24.576 MHz or 12.288 MHz, you
    could first try with the following values.
  */
#ifdef USE_CLOCK_DOUBLER
  SendCmd(SCI_CLOCKF,  0x9800);		/* CLOCKF = 2 x 12.288 MHz */
#else
  SendCmd(SCI_CLOCKF,  0x3000);		/* CLOCKF = 24.576 MHz */
#endif
  SendCmd(SCI_AUDATA,  0x1F40);		/* f_s = 8000 Hz */
  MyWait(100);				/* Wait 100 us */

  /* Load ADPCM patch */
  SendCmdFile(port, "load_ima.cmd");

  /* Activate ADPCM patch */
  SendCmd(SCI_WRAMADDR,0x8021);		/* JMPI 0x400,(I6)+1 */
  SendCmd(SCI_WRAM,    0x2a01);  
  SendCmd(SCI_WRAM,    0x000e);  

  /* Until CTRL-C is pushed */
  while (1) {

    /* Wait for buffer fill to be good */
    while((readWords = ReadCmd(port, 3, SCI_BUF_FILL)) < 128 || readWords >896)
      if (readWords < 128)
	MyWait(1000);	/* Wait 1 ms */

    while (readWords--) {
      /* 2000 of these reads per second if f_s = 8 kHz */
      t = ReadCmd(port, 3, SCI_DATA);

      /* Make sure bytes are ordered in an endian-independent way */
      buf[bufPos++] = t >> 8;
      buf[bufPos++] = t & 0xFF;

      /* Empty buffer once each block. Other strategies may apply as
	 IMA blocks probably don't coinside with disk blocks */
      if (bufPos >= 256) {
	if (buf[3]) {
	  printf("Warning: Bad IMA byte 0x%02x, should be 0 at offset %04x\n",
		 buf[3], ftell(imaFp)+3);
	}
	fwrite(buf, 256, sizeof(unsigned char), imaFp);
	bufPos = 0;
      }
    }
  }
}
